package bookExamples.ch26Graphics.draw2d;

import bookExamples.ch26Graphics.draw2d.shapes.*;
import gui.ClosableJFrame;
import gui.In;
import gui.run.*;
import j2d.ImageUtils;
import j2d.ShortImageBean;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import java.util.Vector;


public class DrawMain extends JComponent
        implements MouseListener, MouseMotionListener {
    private Font font = new Font("Times", Font.PLAIN, 12);

    private Shapes shapeList = new Shapes(0, 0, 0, 0);
    private int x1, y1;
    private Point p2 = new Point();
    private Point lastPoint = new Point();
    private DJShape lastSelectedDJShape = new Line2d(0, 0, 0, 0);
    private boolean addAShapeMode = true;
    private EditList editList = new EditList();

    public DrawMain() {
        // this stops the update
        // problem we are having with drawing.
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        addMouseMotionListener(this);
        addMouseListener(this);
    }


    private void ray() {
        shapeList.addLineToLastCircle();
        repaint();
    }


    private void erase() {
        shapeList = new Shapes(0, 0, 0, 0);
        editList.erase();
        repaint();
    }


    private RunMenuBar getRunMenuBar() {
        RunMenuBar mb = new RunMenuBar();
        mb.add(getFileMenu());
        mb.add(getColorMenu());
        mb.add(getDrawMenu());
        return mb;
    }

    private void outputToPrinter() {
        ImageUtils.print(this);
    }

    public RunMenu getImageMenu() {
        RunMenu rm = new RunMenu("Image");
        rm.add(new RunMenuItem("[ImageShape{control I}") {
            public void run() {
                setLastSelectedShape(new ImageShape(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("crop image to shape") {
            public void run() {
                cropImage();
            }
        });
        return rm;
    }

    //todo finish this.
    public void cropImage() {
        ImageShape is = shapeList.getImageShape();
        Image img = is.getImage();
        if (img == null) {
            In.message("image not in shape list!");
            return;
        }
        ShortImageBean sib = new ShortImageBean(img);
        int w = sib.getWidth();
        int h = sib.getHeight();
        final short v = 0;
        for (int x = 0; x < w; x++) {
            for (int y = 0; y < h; y++) {
                if (shapeList.contains(x, y)) continue;
                sib.setPixel(x,y,v,v,v);
            }
        }

        ImageUtils.displayImage(sib.getImage(), "selected Image");
    }

    private RunMenu getDrawMenu() {
        RunMenu rm = new RunMenu("[Draw");
        rm.add(new RunCheckBoxMenuItem("add shape", true) {
            public void run() {
                addAShapeMode = getState();
                if (!addAShapeMode)
                    setText("select a shape");
                else
                    setText("add shape");
            }
        });
        rm.add(getImageMenu());
        rm.add(new RunMenuItem("sine wave{alt control N}") {
            public void run() {
                final SineFcn f = new SineFcn(100, 100, 100);
                final FunctionShape lastSelectedJShape
                        = new FunctionShape(x1, y1, p2.x, p2.y,
                        f, 0.01);
                //mediator(f);
                setLastSelectedShape(lastSelectedJShape);
            }
        });
        rm.add(new RunMenuItem("hexagon") {
            public void run() {
                setLastSelectedShape(new Hexagon(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("octagon") {
            public void run() {
                setLastSelectedShape(new Octagon(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("s[piral{alt control P}") {
            public void run() {
                setLastSelectedShape(new Spiral(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("le[ns{alt control L}") {
            public void run() {
                setLastSelectedShape(new Lens2d(x1, y1, p2.x, p2.y));
            }
        });

        rm.add(new RunMenuItem("setRotationTo90") {
            public void run() {
                AffineTransform at = getLastSelectedShape()
                        .getAffineTransform();
                at.setToRotation(90 * Math.PI / 180, x1, y1);
                DrawMain.this.repaint();
            }
        });
        rm.add(new RunMenuItem("[erase{control E}") {
            public void run() {
                erase();
            }
        });

        rm.add(new RunMenuItem("[line{control L}") {
            public void run() {
                setLastSelectedShape(new Line2d(0, 0, 0, 0));
            }
        });
        //
        rm.add(new RunMenuItem("R[hombus2d{control H}") {
            public void run() {
                setLastSelectedShape(new Rhombus2d(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("Pentagon[5{control 5}") {
            public void run() {
                setLastSelectedShape(new Pentagon(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("Sq[uareShape2{control U}") {
            public void run() {
                setLastSelectedShape(new Square2d(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("RtTrian[gle2d{alt T}") {
            public void run() {
                setLastSelectedShape(new RtTriangle2d(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("C[ylinder{control Y}") {
            public void run() {
                setLastSelectedShape(new Cylinder(x1, y1, p2.x, p2.y));
            }
        });
        rm.add(new RunMenuItem("C[one{control K}") {
            public void run() {
                setLastSelectedShape(new Cone2d(x1, y1, p2.x, p2.y));
            }
        });

        rm.add(new RunMenuItem("[triangle{control T}") {
            public void run() {
                setLastSelectedShape(new Triangle2d(0, 0, 0, 0));
            }
        });
        rm.add(new RunMenuItem("[circle{control C}") {
            public void run() {
                setLastSelectedShape(new Circle2d(0, 0, 0, 0));
            }
        });
        rm.add(new RunMenuItem("o[val{control O}") {
            public void run() {
                setLastSelectedShape(new Oval2d(0, 0, 0, 0));
            }
        });
        rm.add(new RunMenuItem("[rect{control 6}") {
            public void run() {
                setLastSelectedShape(new Rect2d(0, 0, 0, 0));
            }
        });
        rm.add(new RunMenuItem("[FillRect2d{control F}") {
            public void run() {
                setLastSelectedShape(new FillRect2d(0, 0, 0, 0));
            }
        });
        rm.add(new RunMenuItem("[string{control s}") {
            public void run() {
                setLastSelectedShape(new String2d(p2.x, p2.y, "String me!"));
            }
        });
        rm.add(new RunMenuItem("r[ay{control R}") {
            public void run() {
                ray();
            }
        });
        return rm;
    }

    private JDialog displayPropertyEditor(PropertyEditor f) {
        if (!(f.supportsCustomEditor())) return null;
        f.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                repaint();
            }
        });
        Component comp = f.getCustomEditor();
        JDialog jd = new JDialog();
        jd.setModal(false);
        Container c = jd.getContentPane();
        c.add(comp);

        jd.setSize(200, 100);
        jd.pack();
        jd.setVisible(true);
        return jd;
    }

    private RunMenu getColorMenu() {
        RunMenu colorMenu = new RunMenu("[Color");
        colorMenu.add(new RunMenuItem("set [background color{alt control c}") {
            public void run() {
                setBackgroundColor();
            }
        });
        colorMenu.add(new RunMenuItem("set [foreground color{alt control f}") {
            public void run() {
                setForegroundColor();
            }
        });

        return colorMenu;
    }

    private void setForegroundColor() {
        getParent().setForeground(In.getColor());
    }

    private void setBackgroundColor() {
        getParent().setBackground(In.getColor());
    }

    private RunMenu getFileMenu() {
        RunMenu fileMenu = new RunMenu("[File");

        fileMenu.add(new RunMenuItem("[print{control p}") {
            public void run() {
                outputToPrinter();
            }
        });

        fileMenu.add(new RunMenuItem("[quit{control q}") {
            public void run() {
                System.exit(0);
            }
        });


        return fileMenu;
    }


    public void mouseDragged(MouseEvent e) {
        e.consume();
        lastPoint.x = p2.x;
        lastPoint.y = p2.y;
        p2.x = e.getX();
        p2.y = e.getY();
        repaint();
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        e.consume();
        x1 = e.getX();
        y1 = e.getY();
        p2.x = -1;
        repaint();
    }

    public void mouseReleased(MouseEvent e) {
        if (!addAShapeMode) return;
        e.consume();
        int x2 = e.getX();
        int y2 = e.getY();
        DJShape lastSelectedShape = getLastSelectedShape();
        RunAffinePanel rap;
        //this.removeMouseListener(rap);
        //this.removeMouseMotionListener(rap);
        DJShape djShape = lastSelectedShape.getInstance(x1, y1, x2, y2);
        rap = djShape.getRunAffinePanel();
        Point pc = djShape.getCenterPoint();
        rap.setCenterPoint(pc.x, pc.y);
        //addMouseMotionListener(rap);
        //addMouseListener(rap);
        shapeList.addShape(djShape);
        lastPoint.x = -1;
        p2.x = -1;
        repaint();
    }

    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(MouseEvent e) {
        repaint();
    }

    public void mouseClicked(MouseEvent e) {
        if (!(e.getClickCount() == 2)) return;

        DJShape s = shapeList.getClosestShape(e.getPoint());
        Point pc = s.getCenterPoint();
        RunAffinePanel rap = s.getRunAffinePanel();
        rap.setCenterPoint(pc.x, pc.y);
        s.setSelected(true);
        if (!(s instanceof PropertyEditor)) {
            In.message("no prop editor for:" + s);
            return;
        }

        PropertyEditor propEd = (PropertyEditor) s;

        JDialog jd = displayPropertyEditor(propEd);
        if (jd != null) editList.push(s, jd);
        //System.out.println(s);
        repaint();
        e.consume();

    }

    class EditList {
        Vector v = new Vector();

        public void erase() {
            for (int i = 0; i < v.size(); i++) {
                ShapeDialog sd = (ShapeDialog) v.elementAt(i);

                sd.jd.setVisible(false);
            }
            v = new Vector();
        }

        class ShapeDialog {
            private JDialog jd;
            private DJShape s;

            ShapeDialog(DJShape s, JDialog jd) {
                this.s = s;
                this.jd = jd;
            }
        }

        public void remove(DJShape s) {
            for (int i = 0; i < v.size(); i++) {
                ShapeDialog sd = (ShapeDialog) v.elementAt(i);
                if (sd.s.equals(s)) {
                    sd.jd.setVisible(false);
                }

            }
        }

        public void push(DJShape s, JDialog jd) {
            v.addElement(new ShapeDialog(s, jd));
        }
    }

    public void drawRubberBand(Graphics g) {
        if (!addAShapeMode) return;
        if (p2.x == -1) return;
        g.setXORMode(getBackground());
        DJShape lastSelectedShape = getLastSelectedShape();
        //removeMouseListener(rapOld);
        //removeMouseMotionListener(rapOld);
        DJShape shapeInstance = lastSelectedShape.getInstance(x1, y1, p2.x, p2.y);
        //addMouseListener(rap);
        //addMouseMotionListener(rap);
        shapeInstance.draw(g);
    }

    public void paintComponent(Graphics g) {

        super.paintComponent(g);
        g.setFont(font);
        shapeList.draw(g);
        drawRubberBand(g);

    }


    public static void main(String args[]) {
        DrawMain main = new DrawMain();
        ClosableJFrame cf = new ClosableJFrame("DocJavaDraw");
        cf.setJMenuBar(main.getRunMenuBar());
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(main, BorderLayout.CENTER);
        cf.setSize(300, 300);
        cf.setVisible(true);
    }


    public DJShape getLastSelectedShape() {
        return lastSelectedDJShape;
    }

    public void setLastSelectedShape(DJShape lastSelectedDJShape) {
        this.lastSelectedDJShape = lastSelectedDJShape;

        repaint();
    }
}













